SELECT
    *,
    (
        SELECT
            count(*)
        FROM
            x
    ),
    ARRAY(
        SELECT
            e
        FROM
            f
    )
FROM
    a
WHERE
    b IN (
        SELECT
            c
        FROM
            d
    ) AND
    EXISTS (
        SELECT
            x
        FROM
            y
    )
