SELECT
    2 BETWEEN 1 AND 3,
    2 BETWEEN 3 AND 1,
    2 NOT BETWEEN 1 AND 3,
    2 BETWEEN SYMMETRIC 3 AND 1,
    2 NOT BETWEEN SYMMETRIC 3 AND 1,
    1 IS DISTINCT FROM NULL,
    NULL IS DISTINCT FROM NULL,
    1 IS NOT DISTINCT FROM NULL,
    NULL IS NOT DISTINCT FROM NULL,
    1.5 IS NULL,
    'null' IS NOT NULL,
    true IS TRUE,
    NULL::pg_catalog.bool IS TRUE,
    false IS NOT TRUE,
    NULL::pg_catalog.bool IS NOT TRUE,
    true IS FALSE,
    NULL::pg_catalog.bool IS FALSE,
    true IS NOT FALSE,
    NULL::pg_catalog.bool IS NOT FALSE,
    true IS UNKNOWN,
    NULL::pg_catalog.bool IS UNKNOWN,
    true IS NOT UNKNOWN,
    NULL::pg_catalog.bool IS NOT UNKNOWN,
    'a' LIKE '%',
    'a' ILIKE '%',
    NULLIF( 'a', 'a' ),
    'a' SIMILAR TO '%'
