(
    (
        SELECT
            1
    )
    UNION
    (
        SELECT
            2
    )
    UNION
    (
        SELECT
            3
    )
)
EXCEPT ALL
(
    (
        SELECT
            4
    )
    INTERSECT
    (
        SELECT
            5
    )
)
