WITH sub1 AS (
    SELECT
        1 AS x
), sub2 ( a, b ) AS (
    SELECT
        2,
        3
), sub3 AS MATERIALIZED (
    SELECT
        4 AS y
)
SELECT
    s1.*,
    s2.*,
    s3.*
FROM
    sub1 AS s1,
    sub2 AS s2,
    sub3 AS s3
